--
-- This file is part of TALER
-- Copyright (C) 2025 Taler Systems SA
--
-- TALER is free software; you can redistribute it and/or modify it under the
-- terms of the GNU General Public License as published by the Free Software
-- Foundation; either version 3, or (at your option) any later version.
--
-- TALER is distributed in the hope that it will be useful, but WITHOUT ANY
-- WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-- A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License along with
-- TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
--

BEGIN;

SELECT _v.register_patch('exchange-0009', NULL, NULL);

SET search_path TO exchange;

CREATE INDEX exchange_tables_by_pending
  ON exchange_tables (table_serial_id)
  WHERE NOT finished;
COMMENT ON INDEX exchange_tables_by_pending
  IS 'Used by exchange_do_create_tables';

CREATE FUNCTION foreign_table_aggregation_transient9()
RETURNS void
LANGUAGE plpgsql
AS $$
DECLARE
  table_name TEXT DEFAULT 'aggregation_transient';
BEGIN
  EXECUTE FORMAT (
    'ALTER TABLE ' || table_name ||
    ' ADD CONSTRAINT ' || table_name || '_foreign_wire_target_h_payto'
    ' FOREIGN KEY (wire_target_h_payto) '
    ' REFERENCES wire_targets (wire_target_h_payto) ON DELETE RESTRICT'
  );
END
$$;

CREATE FUNCTION constrain_table_aggregation_transient9(
  IN partition_suffix TEXT
)
RETURNS VOID
LANGUAGE plpgsql
AS $$
DECLARE
  table_name TEXT DEFAULT 'aggregation_transient';
BEGIN
  table_name = concat_ws('_', table_name, partition_suffix);
  EXECUTE FORMAT (
    'ALTER TABLE ' || table_name ||
    ' ADD CONSTRAINT ' || table_name || '_wire_target_h_payto_and_wtid_unique'
    '   UNIQUE (wire_target_h_payto,wtid_raw)'
  );
END $$;


INSERT INTO exchange_tables
    (name
    ,version
    ,action
    ,partitioned
    ,by_range)
  VALUES
    ('aggregation_transient9'
    ,'exchange-0009'
    ,'foreign'
    ,TRUE
    ,FALSE),
    ('aggregation_transient9'
    ,'exchange-0009'
    ,'constrain'
    ,TRUE
    ,FALSE);

COMMIT;
